//
//  NSData+PKWebSocketCrypto.h
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CommonCrypto/CommonDigest.h>

typedef struct SecKey {
    uint32_t num;
    __unsafe_unretained NSString *key;
} SecKey;

@interface NSData (PKWebSocketCrypto)

+ (NSData *)pk_randomBytesOfLength:(NSUInteger)length;
+ (NSData *)pk_hixie76ChallengeWithFirstKey:(SecKey)aKey1
                                  secondKey:(SecKey)aKey2
                                randomData:(NSData *)aData;
- (NSData *)pk_MD5;
- (NSData *)pk_SHA1;
@end


// Generate Hixie76 compilant key struct as per
// http://tools.ietf.org/html/draft-hixie-thewebsocketprotocol-76#section-4.1
//
// 1. Let /spaces_1/ be a random integer from 1 to 12 inclusive.
//    EXAMPLE: For example, 5.
//
// 2. Let /max_1/ be the largest integer not greater than 4,294,967,295 divided by /spaces_1/.
//    EXAMPLE: Continuing the example, 858,993,459.
//
// 3. Let /number_1/ be a random integer from 0 to /max_1/ inclusive.
//    EXAMPLE: For example, 777,007,543.
//
// 4. Let /product_1/ be the result of multiplying /number_1/ and /spaces_1/ together.
//    EXAMPLE: Continuing the example, 3,885,037,715.
//
// 5. Let /key_1/ be a string consisting of /product_1/, expressed in base ten using the numerals in the range U+0030 DIGIT ZERO (0) to U+0039 DIGIT NINE (9).
//    EXAMPLE: Continuing the example, "3885037715".
//
// 6. Insert between one and twelve random characters from the ranges U+0021 to U+002F and U+003A to U+007E into /key_1/ at random positions.
//    NOTE: This corresponds to random printable ASCII characters other than the digits and the U+0020 SPACE character.
//    EXAMPLE: Continuing the example, this could lead to "P388O503D& ul7{K%gX(%715".
//
// 7. Insert /spaces_1/ U+0020 SPACE characters into /key_1/ at random positions other than the start or end of the string.
//    EXAMPLE: Continuing the example, this could lead to "P388 O503D& ul7 {K%gX( %7  15".
SecKey pkws_hixie76Key (void);

void pkws_hixie76ChallengeNumber(uint32_t number, unsigned char * buf);

