//
//  PKWebSocketHandler-Hixie76.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 16/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import "PKWebSocketHandler-Hixie76.h"
#import "PKWebSocketCrypto.h"
#import "PKHTTPRequest.h"
#import "PKHTTPResponse.h"
#import "AsyncSocket.h"
#import "PKWebSocketFrameParser-Hixie76.h"


@implementation PKWebSocketHandlerHixie76

@synthesize expectedChallenge = _expectedChallenge;

- (id)init {
    self = [super init];
    if (self != nil) {
        self.parser = [[PKWebSocketFrameParserHixie76 alloc] init];
        self.parser.delegate = self;
    }
    return self;
}


#pragma mark - High level methods

- (PKHTTPRequest *)handshakeRequestWithURL:(PKWebSocketURL *)anURL error:(NSError **)outError {
    PKHTTPRequest *request = [super handshakeRequestWithURL:anURL error:outError];
    if (request == nil) return request;
    
    SecKey seckey1 = pkws_hixie76Key();
    SecKey seckey2 = pkws_hixie76Key();

    NSString *key1 = seckey1.key;
    NSString *key2 = seckey2.key;
    NSData *randomData = [NSData pk_randomBytesOfLength:8];

    NSData *challenge = [NSData pk_hixie76ChallengeWithFirstKey:seckey1
                                                      secondKey:seckey2
                                                     randomData:randomData];

    self.expectedChallenge = [challenge pk_MD5];

    [request addValue:key1 forHTTPHeaderField:@"Sec-WebSocket-Key1"];
    [request addValue:key2 forHTTPHeaderField:@"Sec-WebSocket-Key2"];
    [request setBody:randomData];

    return request;
}

- (BOOL)validateHandshakeResponse:(PKHTTPResponse *)aResponse error:(NSError **)outError {
    BOOL valid = [super validateHandshakeResponse:aResponse error:outError];
    if (!valid) return valid;

    NSData *body = aResponse.body;
    valid = [self.expectedChallenge isEqualToData:body];

    return valid;
}

- (NSData *)handshakeResponseFrameBoundary {
    return self.expectedChallenge;
}

@end
