//
//  PKWebSocketHandler-RFC6455.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 16/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import "PKWebSocketHandler-RFC6455.h"
#import "PKWebSocketCrypto.h"
#import "NSData+PKWebSocketBase64.h"
#import "PKHTTPRequest.h"
#import "PKHTTPResponse.h"

static NSString * const kWebSocketMagicGUID = @"258EAFA5-E914-47DA-95CA-C5AB0DC85B11";
static NSString * const kWebSocketProtocolVersion = @"13";

@interface PKWebSocketHandlerRFC6455 ()
@property (nonatomic, strong, readwrite) NSString *expectedChallenge;
@end

@implementation PKWebSocketHandlerRFC6455

@synthesize expectedChallenge = _expectedChallenge;

- (PKHTTPRequest *)handshakeRequestWithURL:(PKWebSocketURL *)anURL error:(NSError **)outError {
    PKHTTPRequest *request = [super handshakeRequestWithURL:anURL error:outError];
    [request addValue:kWebSocketProtocolVersion forHTTPHeaderField:@"Sec-WebSocket-Version"];

    // Create our key
    NSString *key = [[NSData pk_randomBytesOfLength:16] base64Encoding];
    [request addValue:key forHTTPHeaderField:@"Sec-WebSocket-Key"];

    // Validate the key we got from server
    NSString *combined = [key stringByAppendingString:kWebSocketMagicGUID];
    NSData *sha1 = [[combined dataUsingEncoding:NSASCIIStringEncoding] pk_SHA1];
    self.expectedChallenge = [sha1 base64Encoding];

    return request;
}

- (BOOL)validateHandshakeResponse:(PKHTTPResponse *)aResponse error:(NSError **)outError {
    BOOL valid = [super validateHandshakeResponse:aResponse error:outError];
    if (!valid) return valid;

    NSString *challenge = [aResponse valueForHTTPHeaderField:@"Sec-WebSocket-Accept"];
    valid = [self.expectedChallenge isEqualToString:challenge];

    return valid;
}

@end
