//
//  PKWebSocketHandler.h
//  PKWebSocket
//
//  Created by Pavel Kunc on 16/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "PKWebSocketFrameParser.h"

@class AsyncSocket;
@class PKHTTPRequest;
@class PKHTTPResponse;
@class PKWebSocketURL;
@class PKWebSocketFrameParser;
@class PKWebSocketMessage;
@class PKWebSocketHandler;

@protocol PKWebSocketHandlerDelegate <NSObject>
- (void)handler:(PKWebSocketHandler *)parser didChangeStatus:(PKWebSocketStatus)status;
- (void)handler:(PKWebSocketHandler *)parser didFailWithError:(NSError *)error;

- (void)handler:(PKWebSocketHandler *)parser didProcessMessage:(PKWebSocketMessage *)message error:(NSError *)error;
- (void)handlerDidSendMessage;
@end


@interface PKWebSocketHandler : NSObject <PKWebSocketFrameDelegate>

@property (nonatomic, strong, readwrite) PKWebSocketFrameParser *parser;
@property (nonatomic, assign, readwrite) PKWebSocketStatus status;
@property (nonatomic, strong, readwrite) NSDictionary *tlsSettings;
@property (nonatomic, assign, readwrite) NSUInteger timeout;
@property (nonatomic, strong, readonly) PKWebSocketURL *url;

@property (nonatomic, weak, readwrite) id<PKWebSocketHandlerDelegate> delegate;


#pragma mark - High level methods

- (BOOL)connectWithURL:(PKWebSocketURL *)anURL error:(NSError **)outError;
- (void)disconnect;

- (void)sendBinaryFrameWithData:(NSData *)data;
- (void)sendTextFrameWithString:(NSString *)string;
- (void)sendData:(NSData *)data tag:(long)tag;


#pragma mark - Handshake frames

- (BOOL)validateHandshakeResponse:(PKHTTPResponse *)response error:(NSError **)outError;
- (PKHTTPRequest *)handshakeRequestWithURL:(PKWebSocketURL *)anURL error:(NSError **)outError;
- (NSData *)handshakeResponseFrameBoundary;

@end

