//
//  PKHTTPRequest.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 16/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import "PKHTTPRequest.h"

#if TARGET_OS_IPHONE
// Note: You may need to add the CFNetwork Framework to your project
#import <CFNetwork/CFNetwork.h>
#endif


@implementation PKHTTPRequest {
    CFHTTPMessageRef _messageRef;
}

- (void)dealloc {
    if (self->_messageRef) CFRelease(self->_messageRef);
}

- (id)initWithMethod:(NSString *)aMethod URL:(NSURL *)aURL {
    self = [super init];
    if (self) {
        CFStringRef method = (__bridge CFStringRef)[aMethod uppercaseString];
        CFURLRef url = (__bridge CFURLRef)aURL;
        CFHTTPMessageRef message = CFHTTPMessageCreateRequest(kCFAllocatorDefault,
                                                              method,
                                                              url,
                                                              kCFHTTPVersion1_1);
        self->_messageRef = message;

    }
    return self;
}

- (NSData *)asData {
    CFDataRef data = CFHTTPMessageCopySerializedMessage(self->_messageRef);
    return (__bridge_transfer NSData *)data;
}

- (NSString *)asString {
    return [[NSString alloc] initWithData:[self asData] encoding:NSASCIIStringEncoding];
}

- (void)setBody:(NSData *)aBody {
   CFHTTPMessageSetBody(self->_messageRef, (__bridge CFDataRef)aBody);
}

- (NSData *)body {
    CFDataRef body = CFHTTPMessageCopyBody(self->_messageRef);
    return (__bridge_transfer NSData *)body;
}

- (void)addValue:(NSString *)aValue forHTTPHeaderField:(NSString *)aField {
    CFStringRef field = (__bridge CFStringRef)aField;
    CFStringRef value = (__bridge CFStringRef)aValue;
    CFHTTPMessageSetHeaderFieldValue(self->_messageRef, field, value);
}

- (NSString *)valueForHTTPHeaderField:(NSString *)aField {
    CFStringRef field = (__bridge CFStringRef)aField;
    CFStringRef value = CFHTTPMessageCopyHeaderFieldValue(self->_messageRef, field);
    return (__bridge_transfer NSString *)value;
}

@end
