//
//  PKHTTPResponse.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 16/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import "PKHTTPResponse.h"

#if TARGET_OS_IPHONE
// Note: You may need to add the CFNetwork Framework to your project
#import <CFNetwork/CFNetwork.h>
#endif


@implementation PKHTTPResponse {
    CFHTTPMessageRef _messageRef;
}

@synthesize statusCode = _statusCode;
@synthesize body       = _body;

- (void)dealloc {
    if (self->_messageRef) CFRelease(self->_messageRef);
}

- (id)initWithData:(NSData *)aData error:(NSError **)outError {
    self = [super init];
    if (self) {
        CFHTTPMessageRef message = CFHTTPMessageCreateEmpty(kCFAllocatorDefault, FALSE);
        if (!message || !CFHTTPMessageAppendBytes(message, [aData bytes], [aData length])) {
            if (message) CFRelease(message);
            if (outError != NULL) {
                NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:@"Unable to parse handshake response!", NSLocalizedDescriptionKey, nil];
                *outError = [NSError errorWithDomain:PKWebSocketErrorDomain
                                                code:PKWebSocketInvalidResponse
                                            userInfo:userInfo];
            }
            return nil;
        }

        if (CFHTTPMessageIsHeaderComplete(message)) {
            self->_messageRef = message;
        } else {
            if (message) CFRelease(message);
            if (outError != NULL) {
                NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:@"Handshake response has incomplete headers!", NSLocalizedDescriptionKey, nil];
                *outError = [NSError errorWithDomain:PKWebSocketErrorDomain
                                                code:PKWebSocketIncompleteResponseHeaders
                                            userInfo:userInfo];
            }
            return nil;
        }
    }
    return self;
}

- (NSString *)valueForHTTPHeaderField:(NSString *)aField {
    CFStringRef field = (__bridge CFStringRef)aField;
    CFStringRef value = CFHTTPMessageCopyHeaderFieldValue(self->_messageRef, field);
    return (__bridge_transfer NSString *)value;
}

- (NSUInteger)statusCode {
    // Casting it from CFIndex to NSUInteger is not propper, but...
    return (NSUInteger)CFHTTPMessageGetResponseStatusCode(self->_messageRef);
}

- (NSData *)body {
    CFDataRef value = CFHTTPMessageCopyBody(self->_messageRef);
    if (CFDataGetLength(value) > 0) {
        return (__bridge_transfer NSData *)value;
    } else {
        CFRelease(value);
        return nil;
    }
}

@end
