//
//  PKWebSocket.h
//  PKWebSocket
//
//  This code is heavily based on Zimt (https://github.com/esad/zimt) by Esad 
//  Hajdarevic.
//
//  Created by Pavel Kunc on 16/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "PKWebSocketConstants.h"
#import "PKWebSocketHandler.h"

@class PKWebSocket;
@class PKWebSocketHandler;
@class PKWebSocketURL;
@class PKWebSocketMessage;


@protocol PKWebSocketDelegate <NSObject>
- (void)webSocketDidOpen:(PKWebSocket *)webSocket;
- (void)webSocketDidClose:(PKWebSocket *)webSocket;
- (void)webSocket:(PKWebSocket *)webSocket didFailWithError:(NSError *)error;
- (void)webSocket:(PKWebSocket *)webSocket didReceiveMessage:(PKWebSocketMessage *)message;

@optional
- (void)webSocketDidSendMessage:(PKWebSocket *)webSocket;
@end

@interface PKWebSocket : NSObject <PKWebSocketHandlerDelegate>

@property (nonatomic, weak, readwrite)   id<PKWebSocketDelegate> delegate;
@property (nonatomic, strong, readwrite) PKWebSocketHandler *handler;

- (id)initWithHandler:(PKWebSocketHandler *)handler
            delegate:(id<PKWebSocketDelegate>)delegate;

- (void)close;
- (BOOL)openWithURLString:(NSString *)url error:(NSError **)error;

- (void)send:(PKWebSocketMessage *)message;
- (void)sendString:(NSString *)aString;
- (void)sendData:(NSData *)aData;

- (void)setTLSSettings:(NSDictionary *)settings;
- (NSDictionary *)TLSSettings;
- (PKWebSocketURL *)url;

@end
