//
//  PKWebSocketURL.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 18/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import "PKWebSocketURL.h"

NSString * const PKWebSocketURLException      = @"PKWebSocketURLException";
NSString * const PKWebSocketURLScheme         = @"ws";
NSString * const PKWebSocketURLSecureScheme   = @"wss";
const NSUInteger PKWebSocketURLDefaultPort    = 80;
const NSUInteger PKWebSocketURLDefaultTLSPort = 443;

@implementation PKWebSocketURL

- (id)initWithString:(NSString *)URLString relativeToURL:(NSURL *)baseURL {
    self = [super initWithString:URLString relativeToURL:baseURL];
    if (self) {
        if (![self.scheme isEqualToString:PKWebSocketURLScheme] &&
            ![self.scheme isEqualToString:PKWebSocketURLSecureScheme]) {
            [NSException raise:PKWebSocketURLException
                        format:[NSString stringWithFormat:@"Unsupported protocol %@",self.scheme]];
            return nil;
        }
    }
    return self;
}

- (BOOL)isSecure {
    return [self.scheme isEqualToString:PKWebSocketURLSecureScheme];
}

- (NSNumber *)port {
    NSNumber *_port = [super port];
    if ([_port integerValue] == 0) {
        if ([self isSecure]) {
            _port = [NSNumber numberWithInteger:PKWebSocketURLDefaultTLSPort];
        } else {
            _port = [NSNumber numberWithInteger:PKWebSocketURLDefaultPort];
        }
    }
    return _port;
}

- (NSString *)hostForHTTPHeader {
    NSString *_host = [super host];
    NSUInteger _port = [self.port integerValue];
    BOOL portSegmentRequired =
        (![self isSecure] && _port != PKWebSocketURLDefaultPort) ||
        ([self isSecure] && _port != PKWebSocketURLDefaultTLSPort);

    if (portSegmentRequired) {
        _host = [NSString stringWithFormat:@"%@:%d", _host, _port];
    }
    return _host;
}

@end
