//
//  PKWebSocketFrameParser-Hixie76Tests.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#ifndef TESTS_RUNNING
#define TESTS_RUNNING
#endif

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "OCMock.h"
#import "PKWebSocketFrameParser-Hixie76.h"

@interface PKWebSocketFrameHixie76Tests : SenTestCase <PKWebSocketFrameDelegate>
@end

@implementation PKWebSocketFrameHixie76Tests {
    NSString *_expectedString;
}

- (void)setUp {
    [super setUp];
    _expectedString = @"";
}

- (void)tearDown {
    _expectedString = nil;
    [super tearDown];
}


#pragma mark - Helpers

- (NSData *)dataWithSingleByteLength {
    // Frame: Optcode|Length (1)|7 (1 byte)|START|H|e|l|l|o|END
    return [NSData dataWithBytes:"\x80\x01\x07\x00\x48\x65\x6C\x6C\x6F\xFF" length:10];
}

- (NSData *)dataWithMultiByteLength {
    // Frame: Optcode|Length (1)|7 (1 byte)|START|H|e|l|l|o|END
    return [NSData dataWithBytes:"\x80\x01\x07\x00\x48\x65\x6C\x6C\x6F\xFF" length:10];
}

- (NSData *)dataWithPlainText {
    // Frame: START|H|e|l|l|o|P|l|a|i|n|END
    return [NSData dataWithBytes:"\x00\x48\x65\x6C\x6C\x6F\x50\x6c\x61\x69\x6e\xFF" length:12];
}

- (NSData *)dataWithClosingHandshake {
    // Frame: END
    return [NSData dataWithBytes:"\xFF" length:1];
}

- (NSData *)dataWithInvalidFrame {
    // Frame:  á|e|l|l|o|END
    return [NSData dataWithBytes:"\xe1\x65\x6C\x6C\x6F\xFF" length:6];
}

- (NSData *)dataWithNotClosedFrame {
    // Frame:  START|á|e|l|l|o
    return [NSData dataWithBytes:"\x00\xe1\x65\x6C\x6C\x6F" length:6];
}


#pragma mark - PKWebSocketFrameDelegate

- (void)parserNeedsReadDataToData:(NSData *)boundary {}
- (void)parserNeedsReadDataOfLength:(NSUInteger)length {}
- (void)parserNeedsInitiateClosingHandshakeWithData:(NSData *)data {
    NSData *closingPayload = [NSData dataWithBytes:"\xFF\x00" length:2];
    STAssertTrue([data isEqualToData:closingPayload], nil);
}
- (void)parserDidProcessString:(NSString *)string error:(NSError *)error {
    if (string) {
        STAssertEqualObjects(string, _expectedString, nil);
    } else {
        NSLog(@"Error: %@", [error localizedDescription]);
        STAssertNil(string, nil);
    }
}


#pragma mark - Tests

// !!!!!!!!
// Until OCMock or Apple solves issue with the weak references we use
// partialMock on self as delegate and not the mock object for protocol.
// !!!!!!!!

- (void)testProcessTextFrameWithSingleByteLengthCorrectly {
    //id delegateMock = [OCMockObject partialMockForObject:self];
    PKWebSocketFrameParserHixie76 *frame = [[PKWebSocketFrameParserHixie76 alloc] init];
    [frame setDelegate:self];

    //[[delegateMock expect] frame:frame didProcessString:@"Hello" error:nil];
     _expectedString = @"Hello";
    [frame processData:[self dataWithSingleByteLength]];
    //[delegateMock verify];
}

/*
- (void)testProcessTextFrameWithMultiByteLengthCorrectly {
    PKWebSocketFrameHixie76 *frame = [[PKWebSocketFrameHixie76 alloc] init];
    frame.delegate = self;
    [frame processData:[self dataWithMultiByteLength]];
}
*/

- (void)testProcessPlainTextFrameCorrectly {
    //id delegateMock = [OCMockObject partialMockForObject:self];
    PKWebSocketFrameParserHixie76 *parser = [[PKWebSocketFrameParserHixie76 alloc] init];
    [parser setDelegate:self];

    //[[delegateMock expect] frame:frame didProcessString:@"HelloPlain" error:nil];
    _expectedString = @"HelloPlain";
    [parser processData:[self dataWithPlainText]];
    //[delegateMock verify];
}

- (void)testProcessInvalidFrameCorrectly {
    //id delegateMock = [OCMockObject partialMockForObject:self];
    PKWebSocketFrameParserHixie76 *parser = [[PKWebSocketFrameParserHixie76 alloc] init];
    [parser setDelegate:self];

    //[[delegateMock expect] frame:frame didProcessString:@"HelloPlain" error:nil];
    _expectedString = nil;
    [parser processData:[self dataWithInvalidFrame]];
    //[delegateMock verify];
}

- (void)testProcessNotClosedFrameCorrectly {
    //id delegateMock = [OCMockObject partialMockForObject:self];
    PKWebSocketFrameParserHixie76 *parser = [[PKWebSocketFrameParserHixie76 alloc] init];
    [parser setDelegate:self];

    //[[delegateMock expect] frame:frame didProcessString:@"HelloPlain" error:nil];
    _expectedString = nil;
    [parser processData:[self dataWithNotClosedFrame]];
    //[delegateMock verify];
}

- (void)testProcessClosingFrameCorrectly {
    //id delegateMock = [OCMockObject partialMockForObject:self];
    PKWebSocketFrameParserHixie76 *parser = [[PKWebSocketFrameParserHixie76 alloc] init];
    [parser setDelegate:self];

    //[[delegateMock expect] frame:frame initiateClosingHandshakeWithData:closingPayload];
    [parser processData:[self dataWithClosingHandshake]];
    //[delegateMock verify];
}

@end

