//
//  PKWebSocketHandler-Hixie75Tests.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "PKWebSocketHandler-Hixie75.h"
#import "PKWebSocketURL.h"
#import "PKHTTPRequest.h"
#import "PKHTTPResponse.h"


@interface PKWebSocketHandlerHixie75Tests : SenTestCase
@end

@implementation PKWebSocketHandlerHixie75Tests {
    PKWebSocketHandlerHixie75 *_handler;
}

- (void)setUp {
    [super setUp];

    _handler = [[PKWebSocketHandlerHixie75 alloc] init];
}

- (void)tearDown {
    _handler = nil;

    [super tearDown];
}


#pragma mark - Helpers

- (NSString *)validHandshakeRequest {
    return @"GET /demo HTTP/1.1\r\n"
            "Host: example.com\r\n"
            "Upgrade: WebSocket\r\n"
            "Connection: Upgrade\r\n"
            "Origin: http://example.com\r\n"
            "\r\n";
}

- (NSString *)validHandshakeResponse {
    return @"HTTP/1.1 101 Web Socket Protocol Handshake\r\n"
            "Upgrade: WebSocket\r\n"
            "Connection: Upgrade\r\n"
            "\r\n";
}

- (NSString *)invalidHandshakeResponse {
    return @"HTTP/1.1 404 Not Found\r\n"
            "\r\n";
}

- (PKWebSocketURL *)URL {
    return [PKWebSocketURL URLWithString:@"ws://example.com/demo"];
}


#pragma mark - Tests opening handshake

- (void)testCreateVaidOpeningHandshakeRequest {
    PKHTTPRequest *request = [_handler handshakeRequestWithURL:[self URL] error:NULL];
    STAssertNotNil(request, nil);
    STAssertEqualObjects([request valueForHTTPHeaderField:@"Upgrade"], @"WebSocket", nil);
    STAssertEqualObjects([request valueForHTTPHeaderField:@"Connection"], @"Upgrade", nil);
}

- (void)testAcceptsCorrectOpeningHandshakeResponse {
    NSData *data = [[self validHandshakeResponse] dataUsingEncoding:NSASCIIStringEncoding];
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    STAssertTrue([_handler validateHandshakeResponse:response error:NULL], nil);
}

- (void)testDeclinesIvalidOpeningHandshakeResponse {
    NSError *error;
    NSData *data = [[self invalidHandshakeResponse] dataUsingEncoding:NSASCIIStringEncoding];

    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:&error];
    STAssertFalse([_handler validateHandshakeResponse:response error:&error], nil);
}


#pragma mark - Tests closing handshake


#pragma mark - Tests framing

@end

