//
//  PKWebSocketHandler-Hixie76Tests.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "PKWebSocketHandler-Hixie76.h"
#import "PKWebSocketURL.h"
#import "PKHTTPRequest.h"
#import "PKHTTPResponse.h"

@interface PKWebSocketHandlerHixie76Tests : SenTestCase
@end

@implementation PKWebSocketHandlerHixie76Tests {
    PKWebSocketHandlerHixie76 *_handler;
}

- (void)setUp {
    [super setUp];

    _handler = [[PKWebSocketHandlerHixie76 alloc] init];
}

- (void)tearDown {
    _handler = nil;

    [super tearDown];
}


#pragma mark - Helpers

- (NSString *)validHandshakeRequest {
    return @"GET /demo HTTP/1.1\r\n"
            "Host: example.com\r\n"
            "Connection: Upgrade\r\n"
            "Sec-WebSocket-Key2: 12998 5 Y3 1  .P00\r\n"
            "Sec-WebSocket-Protocol: sample\r\n"
            "Upgrade: WebSocket\r\n"
            "Sec-WebSocket-Key1: 4 @1  46546xW%0l 1 5\r\n"
            "Origin: http://example.com\r\n"
            "\r\n"
            "^n:ds[4U";
}

- (NSString *)validHandshakeResponse {
    return @"HTTP/1.1 101 WebSocket Protocol Handshake\r\n"
            "Upgrade: WebSocket\r\n"
            "Connection: Upgrade\r\n"
            "Sec-WebSocket-Origin: http://example.com\r\n"
            "Sec-WebSocket-Location: ws://example.com/demo\r\n"
            "Sec-WebSocket-Protocol: sample\r\n"
            "\r\n";
}

- (NSString *)invalidHandshakeResponse {
    return @"HTTP/1.1 404 Not Found\r\n"
    "\r\n";
}

- (PKWebSocketURL *)URL {
    return [PKWebSocketURL URLWithString:@"ws://example.com/demo"];
}


#pragma mark - Tests opening handshake

- (void)testCreateVaidOpeningHandshakeRequest {
    PKHTTPRequest *request = [_handler handshakeRequestWithURL:[self URL] error:NULL];
    STAssertNotNil(request, nil);
    STAssertEqualObjects([request valueForHTTPHeaderField:@"Upgrade"], @"WebSocket", nil);
    STAssertEqualObjects([request valueForHTTPHeaderField:@"Connection"], @"Upgrade", nil);
    STAssertNotNil([request valueForHTTPHeaderField:@"Sec-WebSocket-Key1"], @"There must be frst key.");
    STAssertNotNil([request valueForHTTPHeaderField:@"Sec-WebSocket-Key2"], @"There must be second key.");
    STAssertNotNil(request.body, @"There must be third key");
}

- (void)testAcceptsCorrectOpeningHandshakeResponse {
    // Initiate handshake exchange
    [_handler handshakeRequestWithURL:[self URL] error:NULL];

    // Add expected challenge
    NSMutableData *data = (NSMutableData *)[[self validHandshakeResponse] dataUsingEncoding:NSASCIIStringEncoding];
    [data appendData:_handler.expectedChallenge];

    // Validate
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    BOOL valid = [_handler validateHandshakeResponse:response error:NULL];
    STAssertTrue(valid, nil);
}

- (void)testDeclinesIvalidOpeningHandshakeResponse {
    NSError *error;
    NSData *data = [[self invalidHandshakeResponse] dataUsingEncoding:NSASCIIStringEncoding];

    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:&error];
    STAssertFalse([_handler validateHandshakeResponse:response error:&error], nil);
}

#pragma mark - Tests closing handshake

#pragma mark - Tests framing

@end

