//
//  PKWebSocketHandler-RFC6455Tests.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "PKWebSocketHandler-RFC6455.h"
#import "PKWebSocketURL.h"
#import "PKHTTPRequest.h"
#import "PKHTTPResponse.h"

@interface PKWebSocketHandlerRFC6455Tests : SenTestCase
@end

@implementation PKWebSocketHandlerRFC6455Tests {
    PKWebSocketHandlerRFC6455 *_handler;
}

- (void)setUp {
    [super setUp];

    _handler = [[PKWebSocketHandlerRFC6455 alloc] init];
}

- (void)tearDown {
    _handler = nil;

    [super tearDown];
}


#pragma mark - Helpers

- (NSString *)validHandshakeRequest {
    return @"GET /demo HTTP/1.1\r\n"
            "Host: example.com\r\n"
            "Upgrade: websocket\r\n"
            "Connection: Upgrade\r\n"
            "Sec-WebSocket-Key: dGhlIHNhbXBsZSBub25jZQ==\r\n"
            "Origin: http://example.com\r\n"
            "Sec-WebSocket-Protocol: chat, superchat\r\n"
            "Sec-WebSocket-Version: 13\r\n"
            "\r\n";
}

- (NSString *)validHandshakeResponse {
    return @"HTTP/1.1 101 Switching Protocols\r\n"
            "Upgrade: websocket\r\n"
            "Connection: Upgrade\r\n"
            "Sec-WebSocket-Accept: s3pPLMBiTxaQ9kYGzzhZRbK+xOo=\r\n"
            "Sec-WebSocket-Protocol: chat\r\n"
            "\r\n";
}

- (NSString *)invalidHandshakeResponse {
    return @"HTTP/1.1 404 Not Found\r\n"
    "\r\n";
}

- (PKWebSocketURL *)URL {
    return [PKWebSocketURL URLWithString:@"ws://example.com/demo"];
}


#pragma mark - Tests opening handshake

- (void)testCreateVaidOpeningHandshakeRequest {
    PKHTTPRequest *request = [_handler handshakeRequestWithURL:[self URL] error:NULL];
    STAssertNotNil(request, nil);
    STAssertNotNil([request valueForHTTPHeaderField:@"Upgrade"],nil);
    STAssertNotNil([request valueForHTTPHeaderField:@"Connection"],nil);
    STAssertNotNil([request valueForHTTPHeaderField:@"Sec-WebSocket-Key"],nil);
    STAssertNotNil([request valueForHTTPHeaderField:@"Sec-WebSocket-Version"],nil);
}

- (void)testAcceptsCorrectOpeningHandshakeResponse {
    // Initiate handshake exchange
    [_handler handshakeRequestWithURL:[self URL] error:NULL];

    // Add expected challenge
    NSData *data = [[self validHandshakeResponse] dataUsingEncoding:NSASCIIStringEncoding];

    // Validate
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    BOOL valid = [_handler validateHandshakeResponse:response error:NULL];
    //STAssertTrue(valid, nil);
}

- (void)testDeclinesIvalidOpeningHandshakeResponse {
    NSError *error;
    NSData *data = [[self invalidHandshakeResponse] dataUsingEncoding:NSASCIIStringEncoding];

    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:&error];
    STAssertFalse([_handler validateHandshakeResponse:response error:&error], nil);
}

#pragma mark - Tests closing handshake

#pragma mark - Tests framing

@end

