//
//  PKHTTPRequestTests.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "PKHTTPRequest.h"

@interface PKHTTPRequestTests : SenTestCase
@end

@implementation PKHTTPRequestTests

- (void)setUp {
    [super setUp];
}

- (void)tearDown {
    [super tearDown];
}


#pragma mark - Helpers

- (NSString *)validRequest {
    return @"GET /demo HTTP/1.1\r\n"
            "\r\n";
}

- (NSString *)validRequestWithHeaders {
    return @"GET /demo HTTP/1.1\r\n"
            "Connection: Upgrade\r\n"
            "Upgrade: WebSocket\r\n"
            "\r\n";
}

- (NSData *)validBody {
    return [@"^n:ds[4U" dataUsingEncoding:NSASCIIStringEncoding];
}

- (NSString *)validRequestWithBody {
    return [[self validRequest] stringByAppendingString:@"^n:ds[4U"];
}

- (NSURL *)URL {
    return [NSURL URLWithString:@"http://www.example.com/demo"];
}


#pragma mark - Tests

- (void)testCreatesRequestFromMethodAndURL {
    PKHTTPRequest *request = [[PKHTTPRequest alloc] initWithMethod:@"GET" URL:[self URL]];
    STAssertNotNil(request, nil);
}

- (void)testRequestAsDataReturnsCorrectRepresentation {
    PKHTTPRequest *request = [[PKHTTPRequest alloc] initWithMethod:@"GET" URL:[self URL]];
    NSData *current = [request asData];
    NSData *expected = [[self validRequest] dataUsingEncoding:NSASCIIStringEncoding];
    STAssertTrue([expected isEqualToData:current], nil);
}

- (void)testRequestAsStringReturnsCorrectRepresentation {
    PKHTTPRequest *request = [[PKHTTPRequest alloc] initWithMethod:@"GET" URL:[self URL]];
    STAssertTrue([[request asString] isEqualToString:[self validRequest]], nil);
}

- (void)testRequestWithHeadersAsStringReturnsCorrectRepresentation {
    PKHTTPRequest *request = [[PKHTTPRequest alloc] initWithMethod:@"GET" URL:[self URL]];
    [request addValue:@"Upgrade" forHTTPHeaderField:@"Connection"];
    [request addValue:@"WebSocket" forHTTPHeaderField:@"Upgrade"];
    STAssertTrue([[request asString] isEqualToString:[self validRequestWithHeaders]], nil);
}

- (void)testRequestWithBodyAsStringReturnsCorrectRepresentation {
    PKHTTPRequest *request = [[PKHTTPRequest alloc] initWithMethod:@"GET" URL:[self URL]];
    [request setBody:[self validBody]];
    STAssertTrue([[request asString] isEqualToString:[self validRequestWithBody]], nil);
}

@end

