//
//  PKHTTPResponseTests.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "PKHTTPResponse.h"

@interface PKHTTPResponseTests : SenTestCase
@end

@implementation PKHTTPResponseTests

- (void)setUp {
    [super setUp];
}

- (void)tearDown {
    [super tearDown];
}


#pragma mark - Helpers

- (NSString *)validResponse {
    return @"HTTP/1.1 101 Switching Protocols\r\n"
            "Upgrade: websocket\r\n"
            "Connection: Upgrade\r\n"
            "Sec-WebSocket-Accept: s3pPLMBiTxaQ9kYGzzhZRbK+xOo=\r\n"
            "Sec-WebSocket-Protocol: chat\r\n"
            "WebSocket-Origin: http://www.google.com\r\n"
            "\r\n";
}

- (NSString *)validResponseWithBody {
    return [[self validResponse] stringByAppendingString:@"8jKS'y:G*Co,Wxa-"];
}

- (NSString *)incompleteResponse {
    return @"HTTP/1.1 101 Switching Protocols\r\n"
            "Connection: Upgra";
}


#pragma mark - Tets

- (void)testReturnsNilForIncompleteHeadersAndEmmitsError {
    NSData *data = [[self incompleteResponse] dataUsingEncoding:NSASCIIStringEncoding];

    NSError *error;
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:&error];
    STAssertNil(response, @"Response should not be created for incomplete headers!");
    STAssertNotNil(error, @"Error information sholud be emmited to error!");
    STAssertEquals(error.domain, PKWebSocketErrorDomain, nil);
}


- (void)testItCreatesResponseFromValidData {
    NSData *data = [[self validResponse] dataUsingEncoding:NSASCIIStringEncoding];

    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    STAssertNotNil(response, @"Response could not be created!");
}

- (void)testStatusCodeReturnsCorrectValue {
    NSData *data = [[self validResponse] dataUsingEncoding:NSASCIIStringEncoding];
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    STAssertTrue(response.statusCode == 101, nil);
}

- (void)testGetsValueForExistingHeaderFieldCaseInsensitive {
    NSData *data = [[self validResponse] dataUsingEncoding:NSASCIIStringEncoding];
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    STAssertEqualObjects([response valueForHTTPHeaderField:@"websocket-origin"], @"http://www.google.com", nil);
}

- (void)testHandlesGetValueForNonExistingHeaderField {
    NSData *data = [[self validResponse] dataUsingEncoding:NSASCIIStringEncoding];
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    STAssertNil([response valueForHTTPHeaderField:@"foo-bar"], nil);
}

- (void)testGetsBodyFromResponse {
    NSData *data = [[self validResponseWithBody] dataUsingEncoding:NSASCIIStringEncoding];
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    STAssertNotNil(response.body, nil);
}

- (void)testHandlesGetBodyFromResponseWithoutBody {
    NSData *data = [[self validResponse] dataUsingEncoding:NSASCIIStringEncoding];
    PKHTTPResponse *response = [[PKHTTPResponse alloc] initWithData:data error:NULL];
    STAssertNil(response.body, nil);
}

@end

