//
//  PKWebSocketTests.m
//  PKWebSocketTests
//
//  Created by Pavel Kunc on 16/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "PKWebSocket.h"
#import "PKWebSocketMessage.h"
#import "PKWebSocketHandler-Hixie76.h"
//#import "PKWebSocketHandler-RFC6455.h"

@interface PKWebSocketTests : SenTestCase <PKWebSocketDelegate>
@end

@implementation PKWebSocketTests {
    PKWebSocketHandler *_handler;
    BOOL _finished;
}

- (void)setUp {
    [super setUp];

    _finished = NO;
    _handler = [[PKWebSocketHandlerHixie76 alloc] init];
    //_handler = [[PKWebSocketHandlerRFC6455 alloc] init];
}

- (void)tearDown {
    _handler = nil;

    [super tearDown];
}

- (NSString *)standardURL {
    return @"ws://echo.websocket.org";
    //return @"ws://localhost:10000";
}

- (NSString *)secureURL {
    return @"wss://echo.websocket.org";
}

- (void)webSocket:(PKWebSocket *)webSocket didFailWithError:(NSError *)error {
    _finished = YES;
    STFail([error localizedDescription]);
}

- (void)webSocketDidOpen:(PKWebSocket *)webSocket {
    [webSocket send:[PKWebSocketMessage messageWithString:@"Ping-Pong"]];
}

- (void)webSocketDidClose:(PKWebSocket *)webSocket {}

- (void)webSocket:(PKWebSocket *)webSocket didReceiveMessage:(PKWebSocketMessage *)message {
    _finished = YES;
    STAssertEqualObjects(@"Ping-Pong", message.string, nil);
}


- (void)testStandardConnectionShouldBeSuccessful {
    NSError *error;
    PKWebSocket *ws = [[PKWebSocket alloc] initWithHandler:_handler delegate:self];
    STAssertTrue([ws openWithURLString:[self standardURL] error:&error],nil);

    NSRunLoop *theRL = [NSRunLoop currentRunLoop];
    // Begin a run loop terminated when the finished it set to true
    NSDate *date = [[NSDate date] dateByAddingTimeInterval:1];
    while (!_finished && [theRL runMode:NSDefaultRunLoopMode beforeDate:date]);
}

- (void)testSecureConnectionShouldBeSuccessful {
    NSMutableDictionary *tlsSettings = [NSMutableDictionary dictionaryWithCapacity:4];
    [tlsSettings setObject:@"echo.websocket.org" forKey:(NSString *)kCFStreamSSLPeerName];

    // Allow expired certificates
    [tlsSettings setObject:[NSNumber numberWithBool:YES]
                    forKey:(NSString *)kCFStreamSSLAllowsExpiredCertificates];

    // Allow self-signed certificates
    [tlsSettings setObject:[NSNumber numberWithBool:YES]
                    forKey:(NSString *)kCFStreamSSLAllowsAnyRoot];

    // In fact, don't even validate the certificate chain
    [tlsSettings setObject:[NSNumber numberWithBool:NO]
                    forKey:(NSString *)kCFStreamSSLValidatesCertificateChain];

    NSError *error;
    PKWebSocket *ws = [[PKWebSocket alloc] initWithHandler:_handler
                                                  delegate:self];
    [ws setTLSSettings:tlsSettings];

    STAssertTrue([ws openWithURLString:[self secureURL] error:&error],nil);

    NSRunLoop *theRL = [NSRunLoop currentRunLoop];
    // Begin a run loop terminated when the finished it set to true
    NSDate *date = [[NSDate date] dateByAddingTimeInterval:1];
    while (!_finished && [theRL runMode:NSDefaultRunLoopMode beforeDate:date]);
}

@end
