//
//  PKWebSocketURLTests.m
//  PKWebSocket
//
//  Created by Pavel Kunc on 17/04/2012.
//  Copyright (c) 2012 Pavel Kunc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <SenTestingKit/SenTestingKit.h>
#import "PKWebSocketURL.h"

@interface PKWebSocketURLTests : SenTestCase
@end

@implementation PKWebSocketURLTests

- (void)setUp {
    [super setUp];
}

- (void)tearDown {
    [super tearDown];
}


#pragma mark - Helpers

- (NSString *)standardURL {
    return @"ws://example.com";
}

- (NSString *)secureURL {
    return @"wss://example.com";
}

- (NSString *)standardWithPortURL {
    return [[self standardURL] stringByAppendingString:@":10000"];
}

- (NSString *)secureWithPortURL {
    return [[self secureURL] stringByAppendingString:@":10000"];
}


#pragma mark - Tests

- (void)testThrowsExceptionWhenAttemptToUseNonWebSocketSchema {
    STAssertThrows([PKWebSocketURL URLWithString:@"http://www.example.com"], nil);
}

- (void)testIsSecureForSecureWebSocketSchema {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self secureURL]];
    STAssertTrue([url isSecure],nil);
}

- (void)testIsNotSecureForNormalWebSocketSchema {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self standardURL]];
    STAssertFalse([url isSecure],nil);
}

- (void)testReturnsDefaultPortIfNotSpecified {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self standardURL]];
    STAssertEquals([[url port] integerValue], 80, nil);
}

- (void)testReturnsDefaultPortForSecureURLsIfNotSpecified {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self secureURL]];
    STAssertEquals([[url port] integerValue], 443, nil);
}

- (void)testReturnsSpecifiedPort {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self standardWithPortURL]];
    STAssertEquals([[url port] integerValue], 10000, nil);
}

- (void)testReturnsSpecifiedPortForSecureURLs {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self secureWithPortURL]];
    STAssertEquals([[url port] integerValue], 10000, nil);
}

- (void)testHostMustNotHavePortForStandardPort {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self standardURL]];
    NSString *host = @"example.com";
    STAssertEqualObjects(host, url.hostForHTTPHeader,nil);
}

- (void)testHostMustHavePortForNonStandardPort {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self standardWithPortURL]];
    NSString *host = @"example.com:10000";
    STAssertEqualObjects(host, url.hostForHTTPHeader,nil);
}

- (void)testHostMustHavePortForNonStandardSecurePort {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self secureWithPortURL]];
    NSString *host = @"example.com:10000";
    STAssertEqualObjects(host, url.hostForHTTPHeader,nil);
}

- (void)testHostMustNotHavePortForStandardSecurePort {
    PKWebSocketURL *url = [PKWebSocketURL URLWithString:[self secureURL]];
    NSString *host = @"example.com";
    STAssertEqualObjects(host, url.hostForHTTPHeader,nil);
}

@end

